#!/usr/bin/env python
import sys
import time

try:
    import pygtk
    #tell pyGTK, if possible, that we want GTKv2
    pygtk.require("2.0")
except:
    #Some distributions come with GTK2, but not pyGTK
    pass
try:
    import gtk
    import gtk.glade
    import gobject
except:
    print "An error occured"
    sys.exit(1)

import gnome.ui
import yum


from misc import GuyumConst
import pkgtree as pkgtree
import repotree as repotree
import depstree as depstree
import yumworker as yumworker
from gyfunctions import *
import yumconftree as yumconftree



'''
# main gui
'''
class GuyumGUI:

    def __init__(self, guyumdir, gladefile):
        ''' constructor '''        
        self.queries = []
        self.repoi = None
        self.initrepoq = None
        self.repositories = []
        self.treerepo = None
        self.yumconfig = None
        self.gladefile=gladefile
        self.guyumdir=guyumdir
        self.setupGUI()
       
       
    def setupGUI(self):
        ''' setup GuYum GUI ''' 
    
        # read gladefile
        self.wTree = gtk.glade.XML( self.gladefile )
        
        #Create our dictionary and connect it
        dic = { "on_guyum_button_quit_clicked" : self.quitall,
                "on_quit2_activate" : self.quitall,
                "on_guyum_button_search_clicked" : self.runGuyumQuery,
                "on_guyum_entry_search_activate" : self.runGuyumQuery,
                "on_guyum_button_install_clicked" : self.runInstall,
                "on_guyum_notebook_query_switch_page" : self.switchQueryPage,
                "on_deps_button_ok_clicked" : self.executeInstall,
                "on_deps_button_cancel_clicked" : self.cancelDeps,
                "on_guyum_button_repoedit_clicked" : self.repoEdit,
                "on_repodlg_button_close_clicked" : self.repoEditClose,
                "on_about2_activate" : self.showAboutDialog,
                "on_yum_log_activate" : self.showLogDialog,
                "on_logdialog_button_close_clicked" : self.closeLogDialog,
                "on_guyum_dialog_wait_button_cancel_clicked" : self.cancelQuery,
                "on_yumconfig_activate" : self.showYumConfig,
                "on_dlgyumconf_btnclose_clicked" : self.closeYumconfDialog,
                "on_yumplugins_activate" : self.showYumPluginsDialog,
                "on_dlgyumplugins_btnclose_clicked" : self.closeYumPluginsDialog,
                "on_cb_packages_changed" : self.setQueryFilter,
                "on_cb_archs_changed" : self.setQueryFilter,
                "on_cb_repos_changed" : self.setQueryFilter,
                "on_guyum_window_main_delete_event" : self.quitall2,
                "on_guyum_window_main_destroy" : self.quitall2 }
        self.wTree.signal_autoconnect(dic)
                
        self.window = self.wTree.get_widget("guyum_window_main")
        (self.wTree.get_widget("guyum_notebook_query")).remove_page(0)

        # add init process as timeout function        
        self.wThread = yumworker.YumWorkerThread(self.wTree, self.guyumdir)
        self.wThread.start()  
              

    def showAboutDialog(self, data):
        ''' show about dialog '''
        aboutdialog = self.wTree.get_widget("aboutdialog")        
        aboutdialog.run()
        aboutdialog.hide()
        
        
    def showLogDialog(self, data):
        ''' show log dialog '''    
        text = "\n".join(self.wThread.journal)
        writeIntoTextBuffer(self.wTree.get_widget("guyum_logdialog_textview"), \
                            text)
        self.wTree.get_widget("guyum_dialog_guyumlog").run()
        
        
    def closeLogDialog(self, data):
        ''' close log dialog '''        
        self.wTree.get_widget("guyum_dialog_guyumlog").hide()
                
               
    def repoEditClose(self, data):
        ''' close repository edit dialog '''        
        (self.wTree.get_widget("guyum_dialog_repo")).hide()
        
     
    def repoEdit(self, data):
        ''' show repositories properties  '''

        if self.treerepo==None:
            self.treerepo = repotree.RepoDialogTree()
            
        repoedit_dialog = self.wTree.get_widget("guyum_dialog_repo")
        repoedit_treeview = self.wTree.get_widget("dlgrepo_treeview_repo")
        repo_treeview = self.wTree.get_widget("guyum_treeview_repo")
        
        selection = repo_treeview.get_selection()        

        (model, iter) = selection.get_selected()
        if iter!=None:
            repoid = model.get(iter, repotree.RepoTree._REPO_ID)[0]
            if repoid!=None:
                self.treerepo.add_columns(repoedit_treeview)
                self.treerepo.add_data(self.wThread.initrepoq, repoid)
                self.treerepo.treeview.show()
                
                repoedit_dialog.connect("delete_event", avoidclosewindow)
                repoedit_dialog.connect("destroy", avoidclosewindow)                                
                
                repoedit_dialog.show()
        else:
            dialog = gtk.MessageDialog(parent = self.window, \
                                        flags = gtk.DIALOG_MODAL, \
                                        type = gtk.MESSAGE_WARNING, \
                                        buttons = gtk.BUTTONS_CLOSE, \
                                        message_format=\
                                            "Please choose a repository first")
            dialog.run()
            dialog.destroy()
      
             

    def showYumConfig(self, data):
        if self.yumconfig==None:
            self.yumconfig = yumconftree.YumconfDialogTree()
    
        yumconfig_dialog = self.wTree.get_widget("guyum_dialog_yumconf")
        yumconf_treeview = self.wTree.get_widget("dlgyumconf_treeview_conf")
        self.yumconfig.add_columns(yumconf_treeview)
        if yum.__version__<"3.2":
            self.yumconfig.add_data(self.wThread.initrepoq.conf)
        else:
            self.yumconfig.add_data(self.wThread.initrepoq._conf)
        
        yumconfig_dialog.connect("delete_event", avoidclosewindow)
        yumconfig_dialog.connect("destroy", avoidclosewindow)        
        
        yumconfig_dialog.show()
        
        
    def closeYumconfDialog(self, data):
        yumconfig_dialog = self.wTree.get_widget("guyum_dialog_yumconf")
        if yumconfig_dialog is not None:
            yumconfig_dialog.hide()
            
        
    def showYumPluginsDialog(self, data):            
        yumplugins_dialog = self.wTree.get_widget("guyum_dialog_yumplugins")
        if yumplugins_dialog is not None:
            yumplugins_dialog.show()

            
    def closeYumPluginsDialog(self, data):
        yumconfig_dialog = self.wTree.get_widget("guyum_dialog_yumplugins")
        if yumconfig_dialog is not None:
            yumconfig_dialog.hide()
    
        
    def cancelDeps(self, data):
        self.wTree.get_widget("guyum_dialog_deps").hide()
        
    
    def executeInstall(self, data):
        ''' install resolved dependencies and packages '''    
        self.wThread.execinstall()
        
                
    def runInstall(self, data):
        ''' package installation: resolve dependencies, set repo etc. '''
        self.wThread.install()  
         
        
    def cancelQuery(self, data):
        self.wThread.cancelquery() 
            
    
    def setQueryFilter(self, combo):
        arch = getcbactiveitem(self.wTree.get_widget("guyum_combo_archs"), 0)
        pkgs = getcbactiveitem(self.wTree.get_widget("guyum_combo_packages"), 1)
        repos = getcbactiveitem(self.wTree.get_widget("guyum_combo_repos"), 0)

        if len(self.wThread.queries)>0:
            pkgtree = self.wThread.queries[self.wTree.get_widget("guyum_notebook_query").get_current_page()].pkgtree
            pkgtree.modelfilter = pkgtree.model.filter_new()
            pkgtree.modelfilter.set_visible_func(pkgtree.visible_cb, (arch, pkgs, repos))
            
            sortmodel = gtk.TreeModelSort(pkgtree.modelfilter)
            pkgtree.treeview.set_model(sortmodel)
            sortmodel.set_sort_column_id(1, gtk.SORT_ASCENDING)                    
        
    
    def switchQueryPage(self, widget, data, page_num):
        ''' switch between notebook tabs '''
        treeview = self.wThread._getnthTreeView(widget, page_num)
        recentquery = self.wThread.queries[page_num]
        # is a package already selected, hold focus, else select first package
        selection = treeview.get_selection()
        if selection==None:
            modeliter = treeview.get_model().get_iter_first()
        else:
            (model, modeliter) = selection.get_selected()
        if modeliter!=None:
            selection.select_iter(modeliter)
        else:
            if treeview.get_model().get_iter_first()!=None:
                selection.select_iter(treeview.get_model().get_iter_first())
            
        recentquery.pkgtree.query_selection_changed(selection, recentquery)
        # reset combos
        (archs, repos)=packagelists(recentquery.packages)
        setcbfromlist(self.wTree.get_widget("guyum_combo_archs"), archs)
        setcbfromlist(self.wTree.get_widget("guyum_combo_repos"), repos)
        self.wTree.get_widget("guyum_combo_packages").set_active(0)

        
        
        
    '''
    #
    # get fields for search
    '''
    def _getSearchFields(self):
        searchfields = []
        if (self.wTree.get_widget("guyum_checkbutton_name").get_active()):
            searchfields.append("name")            
        if (self.wTree.get_widget("guyum_checkbutton_description").get_active()):
            searchfields.append("description")
        return searchfields
        
    '''
    #
    # search callback
    '''
    def runGuyumQuery(self, data):  
        searchtext = (self.wTree.get_widget( "guyum_entry_search" )).get_text()

        self.wThread.search(searchtext, self._getSearchFields())
         
    '''
    #
    # quit
    '''
    def quitall(self, data):
        self.wThread.quit()
        gtk.main_quit()
        
        '''
    #
    # quit
    '''
    def quitall2(self, widget, data=None):
        self.quitall(data)

