import sys
import signal
import re
import fnmatch
import time
import os
import exceptions
import copy
import string

import yum
from filetree import DirTree, addPathToTree

from rpmUtils.arch import getArchList
from rpmUtils.miscutils import formatRequire, compareEVR


def computesize(pkgsizein):
    pkgsize = (int)(pkgsizein)
    computedsize = ""
    if pkgsizein>0:
        if (pkgsize>1024):
            if (pkgsize<(1024*1024)):
                computedsize = "%d kB" % (pkgsize/1024)
            else:
                computedsize = "%d MB" % (pkgsize/1024/1024)
        else:
            computedsize = "%d Bytes" % pkgsize
    return computedsize

'''
#
# Merge local installed and not installed package lists
# into one list
'''
def mergepackages(localpkg, remotepkg):
    retpackage = copy.copy(localpkg)
    retpackage.pkgsize = remotepkg.pkgsize
    retpackage.repoid = remotepkg.repoid
    retpackage.remoteversion = remotepkg.version
    retpackage.remoterelease = remotepkg.release
    retpackage.dispremoteversion = remotepkg.dispremoteversion
    retpackage.state='o'
    return retpackage

'''
#
#
'''
def filterpackages(packagelist):
    retpackages = []
    for pkg in packagelist:
        if pkg.packageid!=None and pkg.packageid!='' and not hasattr(pkg.packageid, 'append'):
            retpackages.append(pkg)
    return retpackages

#
# compares a local package with a remote packages 
def cmppackages(localpkg, remotepkg):
    retpackage = copy.copy(localpkg)
    retpackage.repoid = remotepkg.repoid
    retpackage.pkgsize = remotepkg.pkgsize
    retpackage.remoteversion = remotepkg.version
    retpackage.remoterelease = remotepkg.release
    retpackage.dispremoteversion = remotepkg.dispremoteversion        
    retpackage.packageid = remotepkg.packageid
    cmp = compareEVR((localpkg.epoch, localpkg.version, localpkg.release), (remotepkg.epoch, remotepkg.version, remotepkg.release))
    if cmp==-1:
        retpackage.state='u'            
    if cmp==1:
        retpackage.state='d'
    return retpackage
#
# create a unique packages list
def uniquepackages(localpackages, remotepackages):
    retpackages = []
    localtodel = []
    remotetodel = []
    newpkg = None
    
    for remotepkg in remotepackages:
        for localpkg in localpackages:
            newpkg = None
            if remotepkg.name==localpkg.name:
                if remotepkg.pkgdict['nevra']==localpkg.pkgdict['nevra']:
                    newpkg = mergepackages(localpkg, remotepkg)
                else:
                    newpkg = cmppackages(localpkg, remotepkg)
                localtodel.append(localpkg)                    
                if newpkg!=None:                    
                    retpackages.append(newpkg)
                    newpkg = None
                remotetodel.append(remotepkg)
        
    for pkg in localtodel:
        if pkg in localpackages:
            localpackages.remove(pkg)

    for pkg in remotetodel:
        if pkg in remotepackages:
            remotepackages.remove(pkg)

    return retpackages


class guyumpackage:

    def __init__(self):
        # Attributes from pkg
        self.arch = ''
        self.changelog = ''
        self.compsize = 0
        self.conflicts = []
        self.description = ''
        self.displocalversion = ''
        self.dispremoteversion = ''
        self.epoch = ''
        self.files = ''
        self.filetree = None
        self.group = ''
        self.installtid = ''
        self.licenses = ''
        self.localrelease = ''
        self.localversion = ''        
        self.name = ''        
        self.obsoletes = []
        self.po = []
        self.packageid = ''
        self.pkgsize = 0
        self.provides = []
        self.release = ''        
        self.remoterelease = ''        
        self.remoteversion = ''
        self.repoid = ''
        self.repos = []
        self.requires = []
        self.state = ''
        self.summary = ''
        self.url = ''
        self.vendor = ''
        self.version = ''
        
    def uniquelist(self, list):
        newlist = []
        dict={}
        for i in list:
            dict[i]=''
        newlist = dict.keys()
        newlist.sort()
        return newlist

    def reqlist(self, list):
        newlist = []
        for i in list:
            if string.find(i, 'rpmlib(')!=0:
                newlist.append(i)
        newlist = self.uniquelist(newlist)
        return newlist


    def create_local_changelog(self, pkg):
        changelog_times = pkg.hdr['CHANGELOGTIME']
        changelog_names = pkg.hdr['CHANGELOGNAME']
        changelog_texts = pkg.hdr['CHANGELOGTEXT']
        counter = 0
        changelog_array = []
        if hasattr(changelog_times, 'append'):
            for i in changelog_times:
                newtime = time.strftime("%Y-%m-%d", time.localtime(float(changelog_times[counter])))
                changelog_array.append("%s %s \n%s" % (newtime, changelog_names[counter], changelog_texts[counter]))
                counter=counter+1
        else:
            newtime = time.strftime("%Y-%m-%d", time.localtime(float(changelog_times)))
            changelog_array.append("%s %s \n%s" % (newtime, changelog_names[0], changelog_texts[0]))
        
        return "\n".join(changelog_array)

    ''''''
    def stringfromds(self, ds):
        retlist=[]
        for dataset in ds:
            t=dataset[0]
            values=t.split(" ")[1:]
            retlist.append(" ".join(values))
        return retlist

    '''
    #
    #
    '''
    def _dispversion(self, version, release):
        if release!=None and release!="":
            return "-".join((version, release))
        return version

    '''Create new local package'''
    def localpackage(self, pkg, state):
        # Fill attributes
        self.arch = pkg.arch        
        self.changelog = self.create_local_changelog(pkg)

        self.compsize = '0'
        if pkg.hdr.dsFromHeader('conflicts'):
            self.conflicts=self.stringfromds(pkg.hdr.dsFromHeader('conflicts'))
        self.description = pkg.description
        self.epoch = pkg.epoch
        self.files = pkg.hdr['FILENAMES']
        self.group = pkg.hdr.sprintf("%{GROUP}")
        self.installtid = pkg.hdr.sprintf("%{INSTALLTID}")
        if yum.__version__<"2.8":
            self.licenses = pkg.hdr['license']
        else:
            self.licenses = ''.join(pkg.licenses)
        self.localrelease = pkg.release
        self.localversion = pkg.version    
        self.displocalversion = self._dispversion(pkg.version, pkg.release)
        self.name = pkg.name        
        if pkg.hdr.dsFromHeader('obsoletes'):
            self.obsoletes=self.stringfromds(pkg.hdr.dsFromHeader('obsoletes'))
        self.packageid = pkg.pkgid
        self.pkgsize = pkg.hdr['size']
        if pkg.hdr.dsFromHeader('provides'):
            self.provides=self.stringfromds(pkg.hdr.dsFromHeader('provides'))                
        self.release = pkg.release
        self.repoid = pkg.repoid
        self.repos = []
        self.requires = pkg.requiresList()
        self.state = state
        self.summary = pkg.hdr.sprintf("%{Summary}")
        self.url = pkg.hdr.sprintf("%{URL}")
        
        self.vendor = pkg.hdr.sprintf("%{Vendor}")
        self.version = pkg.version
        # Compute size s.o.
        self.repos.append(pkg.repoid)
        self.conflicts = self.uniquelist(self.conflicts)
        self.obsoletes = self.uniquelist(self.obsoletes)
        self.repos.sort()
        self.requires = self.reqlist(self.requires)
        self.files = self.uniquelist(self.files)
        self.filetree = self._createfiletree()
        self.compsize = "%s" % computesize(pkg.hdr['size'])
        self.buildpkgdict()

    def _createfiletree(self):
        filetree = DirTree()
        for f in self.files:
            addPathToTree(f, filetree)
        elemsorted = filetree.sortTree()
        return elemsorted

    '''Create new remote package'''
    def remotepackage(self, pkg, state):
        # Fill attributes
        self.arch = pkg.arch
        self.compsize = '0'
        self.repos = []
        self.repos.append(pkg.repoid)
        self.repos.sort()
        try:
            for conflict in pkg.returnPrco('conflicts'):
                self.conflicts.append(conflict[0])
        except:
            print "cannot create conflicts"
            
        try:
            for obsolete in pkg.returnPrco('obsoletes'):
                self.obsoletes.append(obsolete[0])
        except:
            print "cannot create obsoletes"
            
        self.packageid = pkg.pkgId

        try:   
            for provide in pkg.returnPrco('provides'):
                self.provides.append(provide[0])
        except:
            print "cannot create provides"

        self.state = state
        self.requires = pkg.requiresList()
        self.requires = self.reqlist(self.requires)

        self.release = pkg.release
        self.remoterelease = pkg.release
        self.remoteversion = pkg.version
        self.dispremoteversion = self._dispversion(pkg.version, pkg.release)
        
        self.repoid = pkg.repoid

        self.epoch = pkg.epoch
        self.name = pkg.name
        self.version = pkg.version
        self.changelog = ""
        # for remote packages is changelog not available (causes download of the "otherdata")
        ###self.changelog = "\n".join(["%s %s \n%s" % (time.strftime("%Y-%m-%d", time.localtime(float(date))), vendor, reason) for date, vendor, reason in pkg.returnChangelog()])
        
        # get package information 
        if yum.__version__<='3.1.2':
            pkginfo = pkg.sack.getPackageDetails(pkg.pkgId).info
            pkgsize = pkg.sack.getPackageDetails(pkg.pkgId).size
                
            self.description = pkginfo['description']
            self.group = pkginfo['group']
            self.licenses = pkginfo['license']
            
            self.pkgsize = pkgsize['package']
            if pkginfo['summary']!=None:        
                self.summary = pkginfo['summary']
            if pkginfo['url']!=None:
                self.url = pkginfo['url']            
            if pkginfo['vendor']!=None:
                self.vendor = pkginfo['vendor']
        else:
            self.description = pkg.description
            self.group = pkg.group
            self.licenses = pkg.license
            self.pkgsize = pkg.packagesize

            if pkg.summary!=None:        
                self.summary = pkg.summary
            if pkg.url!=None:
                self.url = pkg.url            
            if pkg.vendor!=None:
                self.vendor = pkg.vendor
            
        # Compute size s.o.
        self.compsize = "%s" % computesize(self.pkgsize)
        
        self.conflicts = self.uniquelist(self.conflicts)
        self.obsoletes = self.uniquelist(self.obsoletes)
        
        self.buildpkgdict()


    def buildpkgdict(self):
        self.pkgdict= {}
        self.pkgdict['name'] = self.name
        self.pkgdict['nameArch'] = '%s.%s' % (self.name, self.arch)
        self.pkgdict['nameVerRelArch'] = '%s-%s-%s.%s' % (self.name, self.version, self.release, self.arch)
        self.pkgdict['nameVer'] = '%s-%s' % (self.name, self.version)
        self.pkgdict['nameVerRel'] = '%s-%s-%s' % (self.name, self.version, self.release)
        self.pkgdict['envra'] = '%s:%s-%s-%s.%s' % (self.epoch, self.name, self.version, self.release, self.arch)
        self.pkgdict['nevra'] = '%s-%s:%s-%s.%s' % (self.name, self.epoch, self.version, self.release, self.arch)
