import sys

try:
    import pygtk
    #tell pyGTK, if possible, that we want GTKv2
    pygtk.require("2.0")
except:
    #Some distributions come with GTK2, but not pyGTK
    pass

try:
    import gtk
    import gtk.glade
    import gobject
except:
    print "An error occured"
    sys.exit(1)

import gnome.ui
from misc import GuyumConst
import guyumquery
import guyumpackage


def showErrorDialog(message):
    errorDlg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
    			                    message_format=message,
    			                    buttons=gtk.BUTTONS_OK)
    if errorDlg.run():
        errorDlg.destroy()

'''
#
'''
def treeviewSortIDSet(column, data):
    (treeview, colid) = data
    sortmodel = treeview.get_model()
    neworder = gtk.SORT_ASCENDING
    (columnid, sortid) = sortmodel.get_sort_column_id()
    if sortid == gtk.SORT_ASCENDING:
        neworder = gtk.SORT_DESCENDING
    sortmodel.set_sort_column_id(colid, neworder)


'''
#
# write into textview
'''
def writeIntoTextBuffer(textview, text):
    textbuffer = textview.get_buffer()
    startiter = textbuffer.get_start_iter()
    enditer = textbuffer.get_end_iter()
    textbuffer.delete(startiter, enditer)
    
    if text!=None:
        textbuffer.insert(startiter, text, -1)
        
'''
#
#
'''
def appendToTextBuffer(textview, text, withthread=False):
    if withthread:
        gtk.gdk.threads_enter()
    textbuffer = textview.get_buffer()
    enditer = textbuffer.get_end_iter()
    if text!=None:
        textbuffer.insert(enditer, text, -1)
    enditer = textbuffer.get_end_iter()
            
    textview.scroll_to_iter(enditer, 0)
    if withthread:
        gtk.gdk.threads_leave()


'''
#
# write into textview
'''
def clearTextBuffer(textview):
    textbuffer = textview.get_buffer()
    startiter = textbuffer.get_start_iter()
    enditer = textbuffer.get_end_iter()
    textbuffer.delete(startiter, enditer)  

'''
#
# write data list into treeview
'''
def writeListIntoTree(treeview, data):
    TEXT_COLUMN = 0
    treemodel = treeview.get_model()
    if treemodel==None:
        treemodel = gtk.TreeStore(gobject.TYPE_STRING)
        column = gtk.TreeViewColumn("", gtk.CellRendererText(), text=TEXT_COLUMN )
        
        treeview.append_column(column)
        treeview.set_model(treemodel)
        treeview.show()
    else:
        treemodel.clear()
    
    for ds in data:
        iter = treemodel.append(None)
        treemodel.set(iter, TEXT_COLUMN, ds)
    

'''

def writeFileListIntoTree(treeview, data):
    ICON_COLUMN = 0
    TEXT_COLUMN = 1
    treemodel = treeview.get_model()
    if treemodel==None:
        treemodel = gtk.TreeStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING)
        
        renderer = gtk.CellRendererPixbuf()        
        column = gtk.TreeViewColumn( '', renderer, pixbuf = ICON_COLUMN )
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_spacing(2)
        treeview.append_column(column)        
        
        column = gtk.TreeViewColumn("", gtk.CellRendererText(), text=TEXT_COLUMN )        
        treeview.append_column(column)
        treeview.set_model(treemodel)
        treeview.show()
    else:
        treemodel.clear()
        
    i = 0
    for elems in data:        
        for elem in elems:    
            iter = treemodel.append(None)            
            treemodel.set(iter, ICON_COLUMN, gtk.gdk.pixbuf_new_from_file("glade/pixmaps/ok.png"),
                                TEXT_COLUMN, '\t'*i+elem.name )
        i = i + 1
 #
# write data list into treeview
'''                               

def filterPackages(packages):
    localdict = {}
    allpackages = []
    localpackages = []
    
    retpackages = []
    
    for pkg in packages:
        if pkg.repoid=='installed':
            pkgkey = "%s-%s-%s" % (pkg.name, pkg.version, pkg.release)
            localdict[pkgkey] = pkg
        else:
            allpackages.append(pkg)
            
    localpackages = localdict.values()
    
    for localpkg in localpackages:
        toappend = True
        for pkg in allpackages:
            if localpkg.name==pkg.name and localpkg.version==pkg.localversion and localpkg.release==pkg.localrelease:
                toappend = False
                break
        if toappend==True:
            retpackages.append(localpkg)
            
    for pkg in allpackages:
        retpackages.append(pkg)
        
    return retpackages
    

def packagelists(packages):
    ''' get lists for packages: package archs, package repos (for filter view) '''
    retarchs = {}
    retrepos = {}
    for pkg in packages:
        retarchs[pkg.arch]=1
        retrepos[pkg.repoid]=1
    return (retarchs, retrepos)        
        

def setcbfromlist(cb, itemlist):
    ''' create comboboxentry from a list '''
    if cb.get_model() is None:
        model = gtk.ListStore(str)
        cb.set_model(model)
        cb.set_text_column(0)   
    else:
        model = cb.get_model()
        model.clear()
                        
    model.append(["All"])
    for item in itemlist:
        model.append([item])

    cb.set_active(0)

        


def updateprogressbar(progressbar, value, factor, text=None):

    if factor!=-1:                  

        gtk.gdk.threads_enter()  
            
        if progressbar.get_fraction()+value/factor>1.0:
            progressbar.set_fraction(1.0)
        else:
            progressbar.set_fraction(progressbar.get_fraction()+value/factor)            
        setpbartext(progressbar, text)
        gtk.gdk.threads_leave()                    


def updatepbar(progressbar, value, text=None):

    if value!=-1:                  

        gtk.gdk.threads_enter()  
            
        if value>1.0:
            progressbar.set_fraction(1.0)
        else:
            progressbar.set_fraction(value)
            
        setpbartext(progressbar, text)
        gtk.gdk.threads_leave()

            

def setpbartext(progressbar, text):
    if text is not None:
        progressbar.set_text(text)


def avoidclosewindow(dialog, data):
    return True
    
    
def getcbactiveitem(combobox, modelindex):
    ''' returns active item from the combobox '''
    if combobox:
        model = combobox.get_model()
        if model:
            index = combobox.get_active()
            if index:
                return model[index][modelindex]
    return "All"
        
    
