import sys

try:
    import pygtk
    #tell pyGTK, if possible, that we want GTKv2
    pygtk.require("2.0")
except:
    #Some distributions come with GTK2, but not pyGTK
    pass

try:
    import gtk
    import gtk.glade
    import gobject
except:
    print "An error occured"
    sys.exit(1)

import gnome.ui
from misc import GuyumConst
import guyumquery
import guyumpackage
from gyfunctions import *


'''
#
# package treemodel
'''
class PkgTree:
    _PKG_STATE=0
    _PKG_NAME=1
    _PKG_LOCALVERSION=2
    _PKG_REMOTEVERSION=3
    _PKG_SIZE=4
    _PKG_ARCH=5
    _PKG_REPOID=6
    _PKG_PKGID=7
    _PKG_STATESTRING=8
    
            
    def __init__(self, wTree):
        ''' constructor ''' 
        self.model = gtk.TreeStore( gtk.gdk.Pixbuf,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING)
        self.wTree = wTree
        self.modelfilter = None
        
            
    def add_columns( self ):
        ''' add columns to treeview ''' 

        self.treeview = gtk.TreeView()
        self.treeview.set_rules_hint(True)
        
        # state column        
        renderer = gtk.CellRendererPixbuf()        
        column = gtk.TreeViewColumn( '', renderer, pixbuf = self._PKG_STATE )
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_spacing(2)
        self.treeview.append_column(column)
        
        # name column
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn( 'Name', renderer, text=self._PKG_NAME )
        column.set_clickable(True)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_spacing(2)        
        column.set_resizable(True)
        column.connect("clicked", treeviewSortIDSet, (self.treeview, self._PKG_NAME))
        self.treeview.append_column(column)
        
        # localversion column
        column = gtk.TreeViewColumn( 'Local', renderer, text=self._PKG_LOCALVERSION )
        column.set_clickable(True)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_resizable(True)
        column.set_spacing(2)        
        self.treeview.append_column(column)
        
        # remoteversion column
        column = gtk.TreeViewColumn( 'Remote', renderer, text=self._PKG_REMOTEVERSION )
        column.set_clickable(True)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_resizable(True)
        column.set_spacing(2)        
        self.treeview.append_column(column)

        # download size column
        column = gtk.TreeViewColumn( 'Download size', renderer, text=self._PKG_SIZE )
        column.set_clickable(True)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_spacing(2)        
        self.treeview.append_column(column)

        # arch column
        column = gtk.TreeViewColumn( 'Arch', renderer, text=self._PKG_ARCH )
        column.set_clickable(True)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_spacing(2)        
        self.treeview.append_column(column)

        # repoid column
        column = gtk.TreeViewColumn( 'Repo', renderer, text=self._PKG_REPOID )
        column.set_clickable(True)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_spacing(2)        
        column.connect("clicked", treeviewSortIDSet, (self.treeview, self._PKG_REPOID))
        self.treeview.append_column(column)

        # pkgid column
        column = gtk.TreeViewColumn( '', renderer, text=self._PKG_PKGID )
        column.set_visible(False)        
        self.treeview.append_column(column)
        
        # state string column        
        column = gtk.TreeViewColumn( '', renderer, text = self._PKG_STATESTRING )
        column.set_visible(False)
        self.treeview.append_column(column)        
       

    def add_data(self, data, guyumdir):
        ''' add data to treemodel '''
        iter = None
        pixmapsdir = guyumdir+"/glade/pixmaps/"
        for package in data:
            iter = self.model.append(None)
            pixbuf = None
            if package.state=='o':
                pixbuf = gtk.gdk.pixbuf_new_from_file(pixmapsdir+"ok.png")
            elif package.state=='i':
                pixbuf = None
            elif package.state=='u':
                pixbuf = gtk.gdk.pixbuf_new_from_file(pixmapsdir+"update.png")
            elif package.state=='d':
                pixbuf = gtk.gdk.pixbuf_new_from_file(pixmapsdir+"downgrade.png")
            else:
                pixbuf = gtk.gdk.pixbuf_new_from_file(pixmapsdir+"error.png")
                      
            self.model.set(iter,
                           self._PKG_STATE, pixbuf,                           
                           self._PKG_NAME, package.name,
                           self._PKG_LOCALVERSION, package.displocalversion,
                           self._PKG_REMOTEVERSION, package.dispremoteversion,
                           self._PKG_SIZE, package.compsize,
                           self._PKG_ARCH, package.arch,
                           self._PKG_REPOID, package.repoid,
                           self._PKG_PKGID, package.packageid,
                           self._PKG_STATESTRING, package.state )
        sortmodel = gtk.TreeModelSort(self.model)
        self.treeview.set_model(sortmodel)
        sortmodel.set_sort_column_id(self._PKG_NAME, gtk.SORT_ASCENDING)
        

    def _set_package_info(self, repoq, packageid):
        ''' display package data '''
        package = repoq.getnamedpackage(packageid)
        self.tooltips = gtk.Tooltips()
        if package is not None:
            writeIntoTextBuffer(self.wTree.get_widget("guyum_textview_changelog"), package.changelog)
            writeListIntoTree(self.wTree.get_widget("guyum_treeview_conflicts"), package.conflicts)
            writeIntoTextBuffer(self.wTree.get_widget("guyum_textview_package_description"), package.description)
            writeListIntoTree(self.wTree.get_widget("guyum_treeview_files"), package.files)
            #writeFileListIntoTree(self.wTree.get_widget("guyum_treeview_files"), package.filetree)
            
            if package.group!=None and package.group!="(none)":
                (self.wTree.get_widget("guyum_entry_category")).set_text(package.group)
            else:
                (self.wTree.get_widget("guyum_entry_category")).set_text("")
            
            if package.licenses!=None and package.licenses!="(none)":
                (self.wTree.get_widget("guyum_entry_licenses")).set_text(package.licenses)
            else:
                (self.wTree.get_widget("guyum_entry_licenses")).set_text("")
            
            writeListIntoTree(self.wTree.get_widget("guyum_treeview_obsoletes"), package.obsoletes)
            writeListIntoTree(self.wTree.get_widget("guyum_treeview_provides"), package.provides)
            writeListIntoTree(self.wTree.get_widget("guyum_treeview_requires"), package.requires)
            
            writeIntoTextBuffer(self.wTree.get_widget("guyum_textview_summary"), package.summary)
            
            href = self.wTree.get_widget("guyum_href_url")
            
            if package.url!=None and package.url!="(none)":
                text = package.url
                if len(text)>27:
                    text = text[:24]+"..."
                href.set_text(text)
                href.set_url(package.url)
                self.tooltips.set_tip(href, package.url)
            else:
                href.set_text('')
                href.set_url('')                
                
            if package.vendor!=None and package.vendor!="(none)":
                (self.wTree.get_widget("guyum_entry_vendor")).set_text(package.vendor)
            else:
                (self.wTree.get_widget("guyum_entry_vendor")).set_text("")
        else:
            raise NameError, 'Cannot find package information'


    def query_selection_changed(self, selection, repoq):
        ''' '''
        (model, pkgiter) = selection.get_selected()
        if pkgiter!=None:
            pkgid = model.get(pkgiter, self._PKG_PKGID)[0]        
            state = model.get(pkgiter, self._PKG_STATESTRING)[0]
            
            install_button = self.wTree.get_widget( "guyum_button_install" )
            if state=='d' or state=='o':
                install_button.set_sensitive(False)
            else:
                install_button.set_sensitive(True)
                
            self._set_package_info(repoq, pkgid)

        
    def visible_cb(self, model, iter, data):
        # arch
        if data[0]!='All':
            if data[0]!=model.get_value(iter, 5):
                return False
        # repos
        if data[2]!='All':
            if data[2]!=model.get_value(iter, 6):
                return False
        # pkgs

        if data[1]!='All':
            if data[1]=='Available':
                newstate = 'i'
            elif data[1]=='Update':
                newstate = 'u'
            elif data[1]=='Downgrade':
                newstate = 'd'
            else:
                newstate = 'o'

            if model.get_value(iter, 8)==newstate:
                return True
            else:
                return False
        return True            

