import sys

try:
    import pygtk
    #tell pyGTK, if possible, that we want GTKv2
    pygtk.require("2.0")
    import pango
except:
    #Some distributions come with GTK2, but not pyGTK
    pass

try:
    import gtk
    import gtk.glade
    import gobject
except:
    print "An error occured"
    sys.exit(1)

import gnome.ui
from misc import GuyumConst
import guyumquery
from gyfunctions import *
import os
import rpm

def search_for_plugin_rpm(pluginname):
    ''' runs local search in RPM database '''    
    retpackages = []
    ts = rpm.TransactionSet()  
    retstr = None
    pluginrpms = ts.dbMatch('name', ('yum-%s' % pluginname))
    
    for pluginrpm in pluginrpms:
        retstr = pluginrpm['Summary']
        break
        
    return retstr


'''
#
# Yum plugins treemodel
'''
class YumPluginsTree:
    _PLUGIN_ENABLED  = 0
    _PLUGIN_ID = 1
    _PLUGIN_DESC = 2
    _PLUGIN_CONF = 3

    # Static descriptions from http://wiki.linux.duke.edu/YumPlugins
    pluginsdesc = { 'fastestmirror': 'A plugin which very quickly sorts the repositories mirror\nlist by connection speed prior to downloading packages.',
                    'installonlyn': 'A plugin for yum which only leaves n \'kernel\' packages installed\ninstead of infinitely doing installonly',
                    'changelog': 'View changelog deltas before or after updating packages',
                    'protectbase': 'Allows packages from given repositories to be protected from being\noverridden by packages from other repositories',
                    'tsflags': 'Allow specifying extra tsflags such as "test" to yum transactions',
                    'versionlock': 'Allow locking packages to certain versions', 
                    'skip-broken': 'Allow skipping of packages with dependency problems',
                    'refresh-updatesd': 'Tell yum-updatesd to refresh itself via dbus'
    }
            
    
    '''
    # constructor 
    ''' 
    def __init__(self, pluginconfpath):
        self.model = None
        self.treeview = None
        self.pluginconfpath = pluginconfpath
            
    '''
    # add columns to treeview 
    ''' 
    def add_columns(self, treeview):
        if self.treeview == None:
            self.treeview = treeview        
            
            # enabled column
            renderer1 = gtk.CellRendererToggle()
            #renderer1.connect('toggled', self.toggle)
            column = gtk.TreeViewColumn('Enabled', renderer1, active=self._PLUGIN_ENABLED )
            column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
            self.treeview.append_column(column)

            # id column
            renderer2 = gtk.CellRendererText()
            column = gtk.TreeViewColumn( 'Plugin', renderer2, text=self._PLUGIN_ID )
            column.set_visible(False)
            self.treeview.append_column(column)            

            # description column
            renderer3 = gtk.CellRendererText()
            column = gtk.TreeViewColumn( 'Description', renderer3, text=self._PLUGIN_DESC, markup=1 )
            column.set_cell_data_func(renderer3, self.__confvalue_data_func)
            self.treeview.append_column(column)

            # conffile column
            renderer4 = gtk.CellRendererText()
            column = gtk.TreeViewColumn( 'Conffile', renderer4, text=self._PLUGIN_CONF )
            column.set_visible(False)
            self.treeview.append_column(column)            


    
    def __confvalue_data_func(self, column, cell, model, iter):
        pluginname = model.get_value(iter, 1)
        plugindesc = model.get_value(iter, 2)
        pluginconf = model.get_value(iter, 3)
        markup = "<span size='large'><b>%s</b></span>\n%s\n<span size='small'><i>%s</i></span>" % (pluginname, plugindesc, pluginconf)
        cell.set_property("markup", markup)        
        
    '''
    #
    # toggle "enabled"-toggle-button event
    '''
    def toggle(self, cell, path):
        iter = self.model.get_iter_from_string(path)
        self.model.set(iter, self._PLUGIN_ENABLED,
                       not self.model.get_value(iter, self._PLUGIN_ENABLED))

    '''
    #
    # add data to treemodel
    '''
    def add_data(self, data):
        if self.model!=None:
            self.model = None
            
        self.model = gtk.TreeStore( gobject.TYPE_BOOLEAN, 
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING )
        iter = None
        for item in data:
            iter = self.model.append(None)
            # is the plugin enabled ?
            enabled=False
            if item['enabled']=='1':
                enabled=True
            # lookup for plugin config file
            conffilename = "Configuration file was not found"
            for dir in self.pluginconfpath:
                conffilename = os.path.join(dir, item['id']+".conf")
                if os.access(conffilename, os.R_OK):
                    break

            # lookup for plugin's RPM, set description
            # from RPM's summary or from local array
            # Default is 'Description is not available'
            plugindesc = search_for_plugin_rpm(item['id'])
            if plugindesc is not None:
                desc = plugindesc
            else:
                if YumPluginsTree.pluginsdesc.has_key(item['id']):
                    desc = YumPluginsTree.pluginsdesc[item['id']]
                else:
                    desc = "Description is not available"
                
            self.model.set(iter,
                           self._PLUGIN_ENABLED, enabled,
                           self._PLUGIN_ID, item['id'],
                           self._PLUGIN_DESC, desc,
                           self._PLUGIN_CONF, conffilename)
        self.treeview.set_model(self.model)
        self.model.set_sort_column_id(self._PLUGIN_ID, gtk.SORT_ASCENDING)        
        
    '''
    #
    # get enabled plugins as list
    '''
    def get_enabled_plugins( self ):
        plugins_enabled = []
        iter = self.model.iter_children( None )
        while iter:
            if not self.model.get_value(iter, self._PLUGIN_ENABLED):
                plugins_enabled.append(self.model.get_value(iter, self._PLUGIN_ID))
            iter = self.model.iter_next(iter)
        return plugins_enabled

