import sys
import time
import os

try:
    import pygtk
    #tell pyGTK, if possible, that we want GTKv2
    pygtk.require("2.0")
except:
    #Some distributions come with GTK2, but not pyGTK
    pass

try:
    import gtk
    import gtk.glade
    import gobject
    import pango
except:
    print "An error occured"
    sys.exit(1)

import gnome.ui
from misc import GuyumConst
import guyumquery
import guyumpackage
import guyuminstall
import pkgtree
import repotree
import depstree
import pluginstree

import yum
import yum.config

if yum.__version__<"3.0":
    from yum.logger import Logger
    import guyumlogger
else:
    import logging
    import guyumlogger3 as guyumlogger


import threading
import Queue
import traceback

from gyfunctions import *

class YumWorkerThread(threading.Thread):

    waitcursor = gtk.gdk.Cursor(gtk.gdk.WATCH)
    defaultcursor = gtk.gdk.Cursor(gtk.gdk.LEFT_PTR)   
    pkgstates = { 'All':'empty.png', 'Installed':'ok.png', 'Available':'empty.png', 'Update':'update.png', 'Downgrade':'downgrade.png' }
    pkgkeys = ('All', 'Installed', 'Available', 'Update', 'Downgrade')

    def __init__(self, wTree, guyumdir):
        threading.Thread.__init__(self)
        self.wTree = wTree
        self.queue = Queue.Queue()
        self.initrepoq = None
        self.repositories = None
        self.queries = []
        self.queue.put(("init", None)) 
        self.journal = []
        self.guyumdir = guyumdir
        self.pluginstree = None
        self.querycancelled = False

        
    def search(self, searchstring, fields):
        ''' Add search command to queue '''
        self.queue.put(("search", (searchstring, fields  )))

        
    def install(self):
        ''' Add install command to queue '''
        self.queue.put(("install", None))


    def execinstall(self):
        ''' Add execute install command to queue '''
        self.queue.put(("execinstall", None))


    def quit(self):
        ''' Add quit command to queue '''
        self.queue.put(("stop", None)) 
        
        
    def cancelquery(self):
        if not self.querycancelled:
            text = "Cancelling query"
            appendToTextBuffer(self.wTree.get_widget("searchwait_details_textview"), text+"\n")
            self.journalAppend(text)
            self.querycancelled = True

        
    def journalAppend(self, text):
        self.journal.append("[ %s ] %s" % (time.strftime("%x %X", time.localtime()), text))
        
        
    def _initGuyum( self ):
        ''' initialize yum, create repository treeview etc. '''
        try:
            self.initrepoq = guyumquery.GuyumQuery(logcb=self.querylogcb)
        except Exception, e:
            self.errorcb("Error initializing GuYum", e)
        else:
            self.repositories = self.initrepoq.getrepos()
            self._createRepoTreeView(self.repositories)
            self._createYumPlugins(self.initrepoq.getplugins())
            gtk.gdk.threads_enter()
            self.wTree.get_widget("guyum_dialog_init").hide()
            self._initPkgsFilterCombo()
            #self.wTree.get_widget("guyum_combo_packages").set_active(0)

            gtk.gdk.threads_leave()
                
        
    def _createRepoTreeView(self, repositories):
        ''' create repositories treeview '''
        self.repotree = repotree.RepoTree()
        gtk.gdk.threads_enter()
        self.repotree.add_columns(self.wTree.get_widget("guyum_treeview_repo"))
        self.repotree.add_data(repositories)        
        gtk.gdk.threads_leave()
        
        
    def _initPkgsFilterCombo(self):        
        cb = self.wTree.get_widget("guyum_combo_packages")

        treestore = gtk.TreeStore(gtk.gdk.Pixbuf,  gobject.TYPE_STRING )

        cell = gtk.CellRendererPixbuf()          
        cb.pack_start(cell, False)  
        cb.add_attribute(cell, 'pixbuf', 0)

        cell = gtk.CellRendererText()  
        cb.pack_start(cell, False)  
        cb.add_attribute(cell, 'text', 1)
        iter = None
        pixmapsdir = self.guyumdir+"/glade/pixmaps/"
        for pkgstate in self.pkgkeys:                   
            iter = treestore.append(None)
            if self.pkgstates[pkgstate] is not None:
                pixbuf = gtk.gdk.pixbuf_new_from_file(pixmapsdir+self.pkgstates[pkgstate])
            else:
                pixbuf = None
            treestore.set(iter, 0, pixbuf, 1, pkgstate)

        cb.set_model(treestore)
        cb.set_active(0)
        
        
    def _createYumPlugins(self, plugins):
        if yum.__version__<"3.2":
            yumconfig = self.initrepoq.conf
        else:
            yumconfig = self.initrepoq._conf
        
        if hasattr(yumconfig, 'pluginconfpath'):
            pluginconfpath = getattr(yumconfig, 'pluginconfpath')
        else:
            pluginconfpath = getattr(yumconfig, 'pluginpath')

        self.pluginstree = pluginstree.YumPluginsTree(pluginconfpath)
        gtk.gdk.threads_enter()
        self.pluginstree.add_columns(self.wTree.get_widget("dlgyumplugin_treeview_edit"))
        self.pluginstree.add_data(plugins)
        gtk.gdk.threads_leave()
        
        
    def _queryTabLabel(self, repoq, packages, searchtext):
        ''' create tablabel for query tab '''        
        tablabel = gtk.HBox()
        hbox = gtk.HBox()
        
        self.tooltip = gtk.Tooltips()                
        ebox = gtk.EventBox()
        ebox.set_visible_window(False)        
        self.tooltip.set_tip(ebox, searchtext)
        hbox.add(ebox)
        
        ebox.add(tablabel)
        
        tabclosebutton = gtk.Button()
        tabclosebutton.set_relief(gtk.RELIEF_NONE)
        self.tooltip.set_tip(tabclosebutton, "Close tab")        
        image = gtk.image_new_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
        image.set_size_request(9, 9)
        tabclosebutton.add(image)
        if len(searchtext)>15:
            labeltext = searchtext[:12]+"..."
        else:
            labeltext = searchtext

        label = gtk.Label(str=("%s (%d)" % (labeltext, len(packages))))        
        tablabel.pack_start(label, True, False, 0)
        tablabel.pack_start(tabclosebutton, False, False, 0)

        hbox.show_all()        
        tabclosebutton.connect("clicked", self._queryTabClose, repoq)
        
        return hbox
        

    def _addQueryTab(self, repoq, packages, searchtext, position):
        ''' add query tab '''    
        query_notebook = self.wTree.get_widget( "guyum_notebook_query" )
        query_tree = pkgtree.PkgTree(self.wTree)  
        
        # create tab, add treeview and fill with data
        query_tab = gtk.ScrolledWindow()
        query_tab.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)        
        query_tree.add_columns()     
        query_tree.add_data( packages, self.guyumdir )
        query_tree.treeview.show()
        query_tab.add(query_tree.treeview)	
        query_tab.show()
        
        selection = query_tree.treeview.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        selection.connect("changed", query_tree.query_selection_changed, repoq)
        repoq.pkgtree = query_tree
        
        return query_notebook.insert_page(query_tab, self._queryTabLabel(repoq, packages, searchtext), position)


    def _queryTabClose(self, widget, data):
        ''' close query tab '''
        i = self.queries.index(data)
        if i>=0:
            self.queries.remove(data)
            query_notebook = self.wTree.get_widget("guyum_notebook_query")
            query_notebook.remove_page(i) 
            if len(self.queries)==0:
                self.wTree.get_widget("guyum_label_startsearch").show()  
                self.wTree.get_widget("guyum_expander_filterview").hide()                  
        else:
            self.errorcb("Unexpected error: cannot close tab")
            
            
    def _runSearch( self, searchstring, fields):
        ''' search for searchstring in fields '''
        # local variables
        query_notebook = self.wTree.get_widget( "guyum_notebook_query" )
        details_textview = self.wTree.get_widget("searchwait_details_textview")     
        wait_progressbar = self.wTree.get_widget("dialog_wait_progressbar")
        dialog_wait = self.wTree.get_widget("guyum_dialog_wait")
        allpackages = []
        finalpackages = []
        
        if len(fields)==0:
            self.errorcb("Error: please choose at least one search field")
            return
            
        # get enabled repositories
        enabledrepos = self.repotree.get_enabled_repos()   
                
        if len(enabledrepos)==0:
            self.errorcb("Error: please choose at least one repository")
            return
        
        # get enabled plugins
        plugins = self.pluginstree.get_enabled_plugins()        
        
        # reset progressbar, show wait dialog, set cursor
        gtk.gdk.threads_enter()
        wait_progressbar.set_fraction(0.0)
        writeIntoTextBuffer(details_textview, "")  
        self.wTree.get_widget("guyum_window_main").window.set_cursor(YumWorkerThread.waitcursor)      
        if dialog_wait is not None:
            dialog_wait.connect("delete_event", avoidclosewindow)
            dialog_wait.connect("destroy", avoidclosewindow)            
            dialog_wait.show()
        gtk.gdk.threads_leave()       
               
        # initialize search
        self.journalAppend("Start searching for %s in %s on %s " % (searchstring, fields, enabledrepos))
        try:
            try:
                repoq = guyumquery.GuyumQuery(logcb=self.querylogcb)

            except Exception, e:
                self.errorcb("Error initalizing search", e)
                
            else:            
                self.journalAppend("Initialization successful")
                
                # start search
                for repo in enabledrepos:
                    if not self.querycancelled:
                        gtk.gdk.threads_enter()
                        text = "Searchfing on %s..." % (repo)
                        appendToTextBuffer(details_textview, text+"\n")
                        self.journalAppend(text)
                        gtk.gdk.threads_leave()

                        try:
                            packages = repoq.searchpackage( searchstring, fields, repo, plugins, wait_progressbar, len(enabledrepos))   
                            for pkg in packages:
                                allpackages.append(pkg)

                            gtk.gdk.threads_enter()
                            text = ("%d packages found." % len(packages))
                            appendToTextBuffer(details_textview, text+"\n")
                            self.journalAppend(text)
                            gtk.gdk.threads_leave()

                        except Exception, e:                
                            self.errorcb("Error searching on %s" % repo, e)
                    else:
                        gtk.gdk.threads_enter()
                        text = "Query was cancelled"
                        appendToTextBuffer(details_textview, text+"\n")
                        self.journalAppend(text)
                        gtk.gdk.threads_leave()                                                    

                self.querycancelled = False

                finalpackages = filterPackages(allpackages)
                self.journalAppend("Filtered: %d packages found" % len(finalpackages))            
                
                repoq.packages = finalpackages
                (archs, repos) = packagelists(finalpackages)
                
                # add new tab, show wait dialog
                gtk.gdk.threads_enter()
                setcbfromlist(self.wTree.get_widget("guyum_combo_archs"), archs)
                setcbfromlist(self.wTree.get_widget("guyum_combo_repos"), repos)
                
                # add query object to the list
                self.queries.append( repoq )

                newcurrpage = self._addQueryTab( repoq, finalpackages, searchstring, -1 )

                if dialog_wait is not None:
                    dialog_wait.hide()
                    
                self.wTree.get_widget("guyum_label_startsearch").hide()
                self.wTree.get_widget("guyum_expander_filterview").show()
                query_notebook.set_current_page(newcurrpage)  
                
                gtk.gdk.threads_leave()        

                self.querydone = True     
                self.journalAppend("Search done.\n")  
        finally:
            self.wTree.get_widget("guyum_window_main").window.set_cursor(YumWorkerThread.defaultcursor)                  
        # End


    def _getnthTreeView(self, query_notebook, n):
        ''' get nth treeview '''
        scrolledwin = query_notebook.get_nth_page(n)
        children = scrolledwin.get_children()
        return children[0]

    '''
    #
    # execute installation
    '''
    def _runInstall(self):
        # local variables        
        query_notebook = self.wTree.get_widget("guyum_notebook_query")
        details_textview = self.wTree.get_widget("searchwait_details_textview")
        wait_progressbar = self.wTree.get_widget("dialog_wait_progressbar")
        dialog_wait = self.wTree.get_widget("guyum_dialog_wait")
        dialog_wait.connect("delete_event", avoidclosewindow)
        dialog_wait.connect("destroy", avoidclosewindow)            
        
        wait_progressbar.set_fraction(0.0)
        
        # get package
        currentpage = query_notebook.get_current_page()
        treeview = self._getnthTreeView(query_notebook, currentpage)
        selection = treeview.get_selection()
        (model, iter) = selection.get_selected()
        pkgid = model.get(iter, pkgtree.PkgTree._PKG_PKGID)[0] 
        package = self.queries[currentpage].findpackage(pkgid)

        gtk.gdk.threads_enter()
        self.wTree.get_widget("guyum_window_main").window.set_cursor(YumWorkerThread.waitcursor)              
        text = "Installing package %s " % package
        writeIntoTextBuffer(details_textview, text)        
        dialog_wait.show()        
        self.journalAppend(text)
        gtk.gdk.threads_leave()

        # prepare installation
        try:
            try:
                self.repoi = guyuminstall.GuyumInstall(logcb=self.instlogcb)

                text = "Resolving dependencies... "
                appendToTextBuffer(details_textview, text+"\n")
                self.journalAppend(text)            
                self.repoi.resolvedependencies(package, wait_progressbar, 1)        
                text = "Done. "
                appendToTextBuffer(details_textview, text+"\n")
                self.journalAppend(text)    
                if len(self.repoi.downloadpkgs)<1:
                    raise Exception, 'Nothing to download and install, break. Package already installed?'

            except Exception, e:
                self.errorcb("Error resolving dependencies.", e)            
                gtk.gdk.threads_enter()
                self.wTree.get_widget("guyum_dialog_wait").hide()            
                gtk.gdk.threads_leave()

            else:
                # create dependencies dialog 
                depsdialog = self.wTree.get_widget("guyum_dialog_deps")
                deps_notebook = self.wTree.get_widget("deps_notebook_packages")
                deps_notebook.remove_page(0)

                # set head line
                deps_label_main = self.wTree.get_widget("deps_label_main")
                deps_label_main.set_markup("<b>Installation dependencies for %s</b>" % package)

                # dependencies tree 
                treedeps = depstree.DepsTree(self.wTree)
                depswindow = gtk.ScrolledWindow()
                treedeps.add_columns()
                treedeps.add_data(self.repoi.downloadpkgs)
                treedeps.treeview.show()

                gtk.gdk.threads_enter()        
                depswindow.add(treedeps.treeview)                
                deps_notebook.append_page(depswindow, gtk.Label("Installation dependencies"))
                depswindow.show()
                depsdialog.connect("delete_event", avoidclosewindow)
                depsdialog.connect("destroy", avoidclosewindow)                                
                depsdialog.show()

                self.wTree.get_widget("guyum_dialog_wait").hide()
                self.wTree.get_widget("deps_label_totalamount").set_text("Total download size: %s " % self.repoi.gettotalamount())

                selection = treedeps.treeview.get_selection()
                selection.set_mode(gtk.SELECTION_SINGLE)
                selection.connect("changed", treedeps.pkgtree_selection_changed, self.repoi) 
                selection.select_iter(treedeps.treeview.get_model().get_iter_first())                

                gtk.gdk.threads_leave()
            
        finally:
            self.wTree.get_widget("guyum_window_main").window.set_cursor(YumWorkerThread.defaultcursor)                  
        
        
    def instlogcb(self, msg=None, pbar1fract=-1, pbar1factor=-1, pbar2fract=-1, pbar1text=None, pbar2text=None):
        ''' installation logging callback '''
        pbar = self.wTree.get_widget("installwait_pbar_total")
        pbar2 = self.wTree.get_widget("installwait_pbar_package")        
        textview = self.wTree.get_widget("installwait_textview") 
        
        if pbar1fract!=-1:        
            updateprogressbar(pbar, pbar1fract, pbar1factor, pbar1text)
        if msg is not None:
            appendToTextBuffer(textview, msg+"\n", True);
            self.journalAppend(msg)
        if pbar2fract!=-1:
            updatepbar(pbar2, pbar2fract, pbar2text)


    def querylogcb(self, msg=None, pbar1fract=-1, pbar1factor=-1, pbar1text=None):
        ''' search logging callback '''
        pbar = self.wTree.get_widget("dialog_wait_progressbar")
        textview = self.wTree.get_widget("searchwait_details_textview")         
        if pbar1fract!=-1:        
            updateprogressbar(pbar, pbar1fract, pbar1factor, pbar1text)
        if msg is not None:
            if msg[-1]!='\n':
                msg+="\n"
            appendToTextBuffer(textview, msg, True);
            self.journalAppend(msg)
        

    def errorcb(self, msg=None, exc=None):
        ''' error callback '''
        if msg is not None:
            self.journalAppend(msg)
        if exc is not None:
            self.journalAppend("%s " % exc)
            traceback.print_exc(file=sys.stderr)

        if msg is not None:            
            gtk.gdk.threads_enter()
            dialog = gtk.MessageDialog(parent = self.wTree.get_widget("guyum_window_main"), \
                                        flags = gtk.DIALOG_MODAL, \
                                        type = gtk.MESSAGE_WARNING, \
                                        buttons = gtk.BUTTONS_CLOSE, \
                                        message_format=msg +"\nFor error details, consult log")
            dialog.run()
            dialog.destroy()
            gtk.gdk.threads_leave()   
                     

    '''
    #
    #    
    '''
    def _executeInstall(self):
        # local variables
        
        wait_progressbar = self.wTree.get_widget("installwait_pbar_total")
        dialog_wait = self.wTree.get_widget("guyum_dialog_installwait")
        query_notebook = self.wTree.get_widget("guyum_notebook_query")  
        details_textview = self.wTree.get_widget("installwait_textview")      
        install_pbar = self.wTree.get_widget("installwait_pbar_package")        
        
        # reset widgets
        wait_progressbar.set_fraction(0.0)    
        updatepbar(install_pbar, 0.01, '')
        gtk.gdk.threads_enter()
        self.wTree.get_widget("guyum_window_main").window.set_cursor(YumWorkerThread.waitcursor)                      
        writeIntoTextBuffer(details_textview, "")
        if dialog_wait is not None:
            dialog_wait.connect("delete_event", avoidclosewindow)
            dialog_wait.connect("destroy", avoidclosewindow)            
            dialog_wait.show()
            
        gtk.gdk.threads_leave()        
        
        text = "Installing package... \n"
        appendToTextBuffer(details_textview, text)
        self.journalAppend(text)
        
        # install package 
        try:
            try:
                self.repoi.installpackage()
            except Exception, e:
                if dialog_wait is not None:
                    dialog_wait.hide()
                    
                self.wTree.get_widget("guyum_dialog_deps").hide()
                self.errorcb("Error install package.", e)
            else:

                text = "Package(s) succesfully installed.\n"
                appendToTextBuffer(details_textview, text)
                self.journalAppend(text)
                    
                gtk.gdk.threads_enter()

                if self.wTree.get_widget("guyum_dialog_installok").run():
                    self.wTree.get_widget("guyum_dialog_installok").hide()
                    if dialog_wait is not None:
                        dialog_wait.hide()
                gtk.gdk.threads_leave()
                
                # replace current page, rerun search in rpmdb    
                currentpage = query_notebook.get_current_page()
                currentquery = self.queries[currentpage]
                rpmdbpkgs=[]
                for pkg in currentquery.packages:
                    rpmdbpkgs.append((pkg.name, pkg.version, pkg.remoteversion, \
                                        pkg.packageid, pkg.release, \
                                        pkg.remoterelease))
                    
                try:
                    result = currentquery.rpmdbsearch(rpmdbpkgs)
                except Exception, e:
                    self.errorcb("Error run RPM-DB search.", e)
                else:
                    for pkg in currentquery.packages:
                        for newpkg in result:
                            if pkg.packageid==newpkg[5]:
                                pkg.state=newpkg[3]
                                pkg.localversion=newpkg[1]
                                pkg.localrelease=newpkg[2]
                                pkg.displocalversion = pkg._dispversion(pkg.localversion, pkg.localrelease)
                                break
                    
                    gtk.gdk.threads_enter()                
                    query_notebook.remove_page(currentpage)
                    newcurrpage = self._addQueryTab(currentquery, currentquery.packages, \
                                                    currentquery.searchstring, \
                                                    currentpage)                                                
                    query_notebook.set_current_page(currentpage)
                    self.wTree.get_widget("guyum_combo_packages").set_active(0)
                    self.wTree.get_widget("guyum_combo_archs").set_active(0)
                    self.wTree.get_widget("guyum_combo_repos").set_active(0)
                    
                    self.wTree.get_widget("guyum_dialog_deps").hide()
                    gtk.gdk.threads_leave()        
        finally:
            self.wTree.get_widget("guyum_window_main").window.set_cursor(YumWorkerThread.defaultcursor)                  
            
    '''
    #
    '''
    def run(self):
        command = ""
        i = 0
        while 1 and command!='stop':

            queueval = self.queue.get()
            (command, args)=queueval
            if command=='stop':
                self.stop = True

            # start search
            if command=='search':
                (searchstring, fields)=args
                self._runSearch(searchstring, fields)
                    
            if command=='init':
                self._initGuyum()
                
            if command=='install':
                self._runInstall()
                
            if command=='execinstall':
                self._executeInstall()
                
            self.stopcommand = False
